// Simple pulse sensor reading 
//  S.o.S. Sense Of Self 
// TeZ, May 2019

#include <WiFi.h>
#include <WiFiUdp.h>
#include <OSCMessage.h>

////////////////////////////////////////////////  
int WID = 9; // ID# OF THIS ESP32 BOARD 
int SENDER = 7;  // here write the number of the sender you want to respond to
const char* BOARDMODE = "receiver";
// const char* BOARDMODE = "sender";
////////////////////////////////////////////////  
//const char* ssid = "TEZNET";
//const char* pass = "teznet23";
const char* ssid = "SenseOfSelf";
const char* pass = "SoS232323";
WiFiUDP Udp;
const IPAddress dest(192, 168, 8, 255);
unsigned int rxport = 12345; // local ESP32 port to listen OSC
unsigned int txport = 54321; // remote port to receive OSC
IPAddress thisip;

int PulseSensorPin = 36; // VP pin on ESP32
int LEDPIN = 2;   //  The on-board LED
int TTGOLEDPIN = 22;
int Signal;  
int Threshold = 3450; 

int device_id = -1;
int WIP[]={0,0,0,0};
int hold = 1;
bool firstime = true;

float lastreading, thisreading;
float IBI = 0;
unsigned long lastbeat = 0;
unsigned long now = 0;
int sensor_id = -1;
bool beatDispatched = false;

float vibration = 100;
bool vibrostate = false;

#include <NeoPixelBus.h>
const uint16_t PixelCount = 1; // how many neopixels in your strip?
const uint8_t PixelPin = 27; // ignored for Esp8266 - Wemos, it uses GPIO2 = PIN D4 (UART MODE)
#define colorSaturation 55
NeoPixelBus<NeoGrbFeature, Neo800KbpsMethod> strip(PixelCount, PixelPin);
RgbColor red(colorSaturation, 0, 0);
RgbColor green(0, colorSaturation, 0);
RgbColor blue(0, 0, colorSaturation);
RgbColor white(colorSaturation);
RgbColor black(0);

////////////////////////////////////
void setup() {

  if(BOARDMODE == "receiver"){
    rxport = 12345; 
    txport = 54321;     
  }else{
    txport = 12345; 
    rxport = 54321;     
  }
  
  pinMode(LEDPIN,OUTPUT); // internal ESP32 led     
  pinMode(TTGOLEDPIN,OUTPUT); // internal TTGO mini 32 led     
 
  strip.Begin(); // init neopixel
  strip.SetPixelColor(0, blue);
  strip.Show(); 
  delay(2000);
  strip.SetPixelColor(0, black); 
  strip.Show();
   
  Serial.begin(115200); 

     // connect to WiFi
      Serial.println();
      Serial.println();
      Serial.print("Connecting to ");
      Serial.println(ssid);     
      WiFi.begin(ssid, pass);    
      while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
      }    
      Serial.println("");
      Serial.println("WiFi connected");  
      Serial.println("IP address: ");
      thisip = WiFi.localIP();
      Serial.println( thisip );

      Udp.begin(rxport);
      Serial.println("Starting UDP");

      for(int n=1; n<=3;n++){
          strip.SetPixelColor(0, green);
          strip.Show(); 
          digitalWrite(LEDPIN,HIGH);  
          digitalWrite(TTGOLEDPIN,HIGH);  
          delay(200);
          strip.SetPixelColor(0, black);
          strip.Show(); 
          digitalWrite(LEDPIN,LOW);  
          digitalWrite(TTGOLEDPIN,LOW);  
          delay(200);    
 
      }


  
}

////////////////////////////////////
void loop() {

  if (BOARDMODE == "sender"){
        senderLoop();
        delay(20);
  }else{
        osc_message_pump();
        delay(20);
  }

}
///////////////////////////////////
void senderLoop(){

  Signal = analogRead(PulseSensorPin);  // Read the PulseSensor’s value.
  // SendSignal(Signal);

  Serial.println(Signal);  

  if(Signal > Threshold && beatDispatched == false){ 
      digitalWrite(LEDPIN,HIGH);
      digitalWrite(TTGOLEDPIN,HIGH);    
       SendBeat(HIGH);
       beatDispatched = true; 
       strip.SetPixelColor(0, red);
       strip.Show(); 
       

  } else  {
     digitalWrite(LEDPIN,LOW); 
     digitalWrite(TTGOLEDPIN,LOW);  
    if(Signal < (Threshold / 1.5)){     
      beatDispatched = false ; 
      strip.SetPixelColor(0, black);
      strip.Show(); 
      delay(10);
      SendBeat(LOW);
     }
  }

}

///////////////////////////////////
void receiverLoop(){

  // nada
  
}
///////////////////////////////////
void SendBeat(bool beatstate) {

     OSCMessage out("/sos/beat");
    out.add(WID);
     if(beatstate == HIGH){
        out.add(1);
     }else{
      out.add(0);
     }
     Udp.beginPacket(dest, txport);
     out.send(Udp);
     Udp.endPacket();
}
///////////////////////////////////
void SendSignal(int mySignal) {

     OSCMessage out("/sos/signal");
     out.add(WID);
     out.add(mySignal);
     Udp.beginPacket(dest, txport);
     out.send(Udp);
     Udp.endPacket();
}
//////////////////////////////////
/////////////////////////////////////
/// READ OSC MESSAGES ///
void osc_message_pump() {  
  OSCMessage in;
  int size;

  if( (size = Udp.parsePacket()) > 0)
  {
    Serial.println("processing OSC package");
    // parse incoming OSC message
    while(size--) {
      in.fill( Udp.read() );
    }

    if(!in.hasError()) {
      in.route("/wemos/ack", on_ack);
      in.route("/sos/beat", on_beat);
    }
  } // if
}



/////////////////////////////////////
void on_beat(OSCMessage &msg, int addrOffset) {

   
   int sender_id;
   float value; 
  if(msg.isInt(0)) {
    sender_id = msg.getInt(0);
    value = msg.getInt(1);
  }

  if(sender_id == SENDER){  // discriminate the source, only receive from selected sender

      if(value){
          strip.SetPixelColor(0, green);
          strip.Show(); 
          digitalWrite(TTGOLEDPIN,HIGH);  
      }else{
          strip.SetPixelColor(0, black);
          strip.Show(); 
          digitalWrite(TTGOLEDPIN,LOW);  
      }


    Serial.print("OSC beat message value: ");
    Serial.println(value);
    


//        vibrostate = !(vibrostate);  
//         if (vibrostate){
//           ledcWrite(0, 100);
//         }else{
//          ledcWrite(0, 0);
//          
//         }

  }


}

/////////////////////////////////////
void on_ack(OSCMessage &msg, int addrOffset) {
 
   int sender_id;
   float value; 
  if(msg.isInt(0)) {
    sender_id = msg.getInt(0);
    value = msg.getInt(1);
  }

    Serial.print("OSC ACK message value: ");
    Serial.println(value);
    


}
